CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  CR = 13

  ' Define Pins
  CS = 0        'CLK = CS + 1, MISO = CS + 2, MOSI = CS + 3  -> ADC PINS
  ACC1 = 4
  ACC2 = 5
  ACC3 = 6
  ACC4 = 7
  WIND = 8
  W_STOP = 9
  E_STOP = 10

  XB_IN = 15
  XB_OUT = 16
  XB_BAUD = 9600

  ' Output
  UP = 11
  DOWN = 12
  LEFT = 13
  RIGHT = 14

  ' Offsets
  east_offset = 0
  west_offset = -19
  up_offset = 10
  down_offset = 0

  ' Define Specifics
  Max_Freq_Count = 4095          ' Max number counted from sensor
    
  SampleRate = 70                    ' Time between E/W and U/D sample
  MaxSampleCount = 1000               ' Cloud Buffer

  Min_EW_move = 164                  ' Minimum Pulse count to move East or West
  Min_UD_move = 164                  ' Minimum Pulse count to move Up or Down
  UD_Park_Range = 162
  EW_Park_Range = 162                ' Go to Park mode when EW pulse count equals X
  Park_Timer = 10000              ' Number of loops before Park is initiated
  Wake_Timer = 10000
  
  max_diff = 5

  ' Serial Terminal Stuff
  DEBUG_BAUD = 9_600
  DEBUG_HEADING_X = 1          ' Output formatting data
  DEBUG_HEADING_Y = 1
  DEBUG_DYNAMIC_X = DEBUG_HEADING_X + 8
  DEBUG_DYNAMIC_Y = DEBUG_HEADING_Y + 2

VAR
  LONG variableStack[400]
  LONG moveStack[400]
  LONG wifiStack[400]
  LONG temp
  LONG EWArray[MaxSampleCount]
  LONG UDArray[MaxSampleCount]
  LONG East_Count, West_Count, Up_Count, Down_Count  
  LONG EWdiff
  LONG EWdiffavg
  LONG UDdiff
  LONG UDdiffavg
  BYTE EastSwitch
  BYTE WestSwitch
  BYTE shutdown
  BYTE Park_Flag
  BYTE threshold_flag
  BYTE moving_flag
  BYTE buffer[200]
  BYTE cog
  
OBJ
  adc           : "ADC_MCP3204_SE_SPI"
  Pst           : "Parallax Serial Terminal"
  xb            : "FullDuplexSerial"

PUB go | index, char, EWarray_pointer, UDarray_pointer, aloopcount, arraysum, threshold_count
  shutdown := 0
  moving_flag := 0
  aloopcount := 0
  threshold_count := 0
  threshold_flag := 0

  dira[E_STOP]~
  dira[W_STOP]~

  cognew(controlMovement, @moveStack)                                    '1 Cog
  pause(500)
  cognew(Handle_Wifi, @wifiStack)                                        '2 Cogs
  pause(500)                                                                 
  Pst.Start(DEBUG_BAUD)                                                         '1 Cog
  pause(300)
  'adc.start(CS, 0, @East_Count, @West_Count, @Up_Count, @Down_Count)            '1 Cog
  pause(1000)
  
  Pst.Dec(cog)
  'Pst.Clear
  Pst.Position(DEBUG_HEADING_X, DEBUG_HEADING_Y)
  Pst.Str(string("Current Position", 13))
  Pst.PositionX(DEBUG_HEADING_X)
  Pst.Str(string("------------------------", 13))
  Pst.PositionX(DEBUG_HEADING_X)
  Pst.Str(string("East  ................", 13))
  Pst.PositionX(DEBUG_HEADING_X)
  Pst.Str(string("West  ................", 13))
  Pst.PositionX(DEBUG_HEADING_X)
  Pst.Str(string("EWdiff ...............", 13))
  Pst.PositionX(DEBUG_HEADING_X)
  Pst.Str(string("EWavg ...............", 13))

  Pst.PositionX(DEBUG_HEADING_X)
  Pst.Str(string("Up  .................", 13))
  Pst.PositionX(DEBUG_HEADING_X)
  Pst.Str(string("Down  ................", 13))
  Pst.PositionX(DEBUG_HEADING_X)
  Pst.Str(string("UDdiff ...............", 13))
  Pst.PositionX(DEBUG_HEADING_X)
  Pst.Str(string("UDavg ...............", 13))
  Pst.PositionX(DEBUG_HEADING_X)
  Pst.Str(string("Park ...............", 13))
  Pst.PositionX(DEBUG_HEADING_X)
  Pst.Str(string("West SW ...............", 13))    
  Pst.PositionX(DEBUG_HEADING_X)
  Pst.Str(string("East SW ...............", 13))
  Pst.PositionX(DEBUG_HEADING_X)
  Pst.Str(string("Thrshld ...............", 13))
  Pst.PositionX(DEBUG_HEADING_X)
  Pst.Str(string("Moving ...............", 13))

  repeat
    ReadSwitches  
    Update_Debug_Window

    if East_Count > West_Count
      EWdiff := East_Count - West_Count
    else
      EWdiff := West_Count - East_Count
'
    if Up_Count > Down_Count
      UDdiff := Up_Count - Down_Count
    else
      UDdiff := Down_Count - Up_Count

    aloopcount++
    
    if aloopcount => SampleRate
      if East_Count > Min_EW_move OR West_Count > Min_EW_move
        EWArray[EWarray_pointer] := EWdiff
        EWarray_pointer++
        if EWarray_pointer == MaxSampleCount
          EWarray_pointer := 0
      
      if Up_Count > Min_UD_move OR Down_Count > Min_UD_move
        UDArray[UDarray_pointer] := UDdiff
        UDarray_pointer++
        if UDarray_pointer == MaxSampleCount
          UDarray_pointer := 0

      EWdiffavg := getAverage(@EWarray)
      UDdiffavg := getAverage(@UDarray)
      
      aloopcount := 0
      
    if East_Count > EW_Park_Range OR West_Count > EW_Park_Range OR Up_Count > UD_Park_Range OR Down_Count > UD_Park_Range
      threshold_count++
      if threshold_count > 2000
        threshold_flag := 1
    else
      threshold_flag := 0
      threshold_count := 0

PUB Handle_Wifi | index, char
  xb.Start(XB_OUT, XB_IN, %0000, 9_600)
  pause(500)
  xb.str(string("Solar Connected!"))
  repeat
    index~                                              ' Reset the buffer pointer
    repeat                                                                                               
      char := xb.rxCheck                                ' Read port
      if char > 31                                      ' ASCII code of character must be > SPACE
        buffer[index++] := char                         ' Append it to the string buffer
    until char == 13                                    ' Received an end of line character
    buffer[index]~                                      ' Insert a null at the end of the string buffer, so it can a zstring
    
PUB controlMovement | park_loopcount, wake_loopcount, moving_east, moving_west, moving_up, moving_down
  dira[LEFT]~~
  dira[RIGHT]~~
  dira[UP]~~
  dira[DOWN]~~

  moving_east := 0
  moving_west := 0
  moving_up := 0
  moving_down := 0
  
  repeat
    if EastSwitch == 1
      outa[RIGHT] := 0

    if WestSwitch == 1
      outa[LEFT] := 0
  
    if EWdiffavg => max_diff
      if East_Count > West_Count AND moving_west == 0
        goEast
        moving_east := 1  
      elseif West_Count > East_Count AND moving_east == 0
        goWest
        moving_west := 1
      else
        EWoff
        moving_east := 0
        moving_west := 0
                
    if UDdiffavg => max_diff
      if Up_Count > Down_Count AND moving_down == 0
        goUp
        moving_up := 1
      elseif Down_Count > Up_Count AND moving_up == 0
        goDown
        moving_down := 1
      else
        UDoff
        moving_down := 0
        moving_up := 0
        
    if East_Count < EW_Park_Range AND West_Count < EW_Park_Range AND Up_Count < UD_Park_Range AND Down_Count < UD_Park_Range AND Park_Flag == 0
      park_loopcount++
      wake_loopcount := 0
      if park_loopcount > Park_Timer
        Park
        Park_Flag := 1
    else                                                                    
      if East_Count > EW_Park_Range OR West_Count > EW_Park_Range OR Up_Count > UD_Park_Range OR Down_Count > UD_Park_Range
        wake_loopcount++
        if Park_Flag == 1 AND moving_flag == 0 AND threshold_flag == 1 AND wake_loopcount > Wake_Timer
          Wake
          wake_loopcount := 0 
      park_loopcount := 0

PUB getAverage(the_array) | i, arraysum
    arraysum := 0
    repeat i from 0 to MaxSampleCount - 1
      arraysum += long[the_array][i]
    return arraysum / MaxSampleCount
     
PUB goEast
  outa[LEFT] := 0
  if EastSwitch == 0
    moving_flag := 1
    outa[RIGHT] := 1
  else
    outa[RIGHT] := 0
  return

PUB goWest
  outa[RIGHT] := 0
  if WestSwitch == 0
    moving_flag := 1
    outa[LEFT] := 1
  else
    outa[LEFT] := 0
  return

PUB goUp
  outa[DOWN] := 0
  moving_flag := 1
  outa[UP] := 1
  return
  
PUB goDown
  outa[UP] := 0
  moving_flag := 1
  outa[DOWN] := 1
  return

PUB EWoff
  outa[LEFT] := 0
  outa[RIGHT] := 0
  pause(100)
  moving_flag := 0
  reset_EW_avg
  return

PUB UDoff
  outa[UP] := 0
  outa[DOWN] := 0
  pause(100)
  moving_flag := 0
  reset_UD_avg
  return

PUB reset_EW_avg
  longfill(@EWarray, 0, MaxSampleCount)
  EWdiffavg := 0
  'pause(1000)
  return

PUB reset_UD_avg
  longfill(@UDarray, 0, MaxSampleCount)
  UDdiffavg := 0
  'pause(1000)
  return
  
PUB ReadSwitches | i
  if ina[W_STOP] == 0
    WestSwitch := 0
  else
    WestSwitch := 1
    
  if ina[E_STOP] == 0
    EastSwitch := 0
  else
    EastSwitch := 1
    
  return

PUB Park | i
  Park_Flag := 1
  threshold_flag := 0
  UDoff
  EWoff
  pause(100)
  i := 0
  'goUp

  repeat until WestSwitch == 1 or threshold_flag == 1
    goWest
  EWoff
  pause(2000)
  
  repeat until i == 1199 or threshold_flag == 1
    goEast
    pause(10)
    i++
  EWoff

  pause(2000)
  
  i := 0
  repeat until i == 30000 or threshold_flag == 1
    goUp
    'goDown
    pause(10)
    i++
  UDoff  
  return

PUB Wake | i
  Park_Flag := 0
  UDoff
  EWoff
  pause(100)

  repeat until EastSwitch == 1
    goEast
    pause(10)
  EWoff

  pause(2000)
  
  i := 0
  repeat until i == 20000 or threshold_flag == 1 and i => 5000
    'goUp
    goDown
    pause(10)
    i++
  UDoff
  return

PUB Update_Debug_Window
    Pst.Position(DEBUG_DYNAMIC_X, DEBUG_DYNAMIC_Y)
    Pst.Dec(East_Count)
    Pst.Str(string("..."))
    Pst.Position(DEBUG_DYNAMIC_X, DEBUG_DYNAMIC_Y + 1)
    Pst.Dec(West_Count)
    Pst.Str(string("..."))
    Pst.Position(DEBUG_DYNAMIC_X, DEBUG_DYNAMIC_Y + 2)
    Pst.Dec(EWdiff)
    Pst.Str(string("..."))
    Pst.Position(DEBUG_DYNAMIC_X, DEBUG_DYNAMIC_Y + 3)
    Pst.Dec(EWdiffavg)
    Pst.Str(string("..."))
    Pst.Position(DEBUG_DYNAMIC_X, DEBUG_DYNAMIC_Y + 4)
    Pst.Dec(Up_Count)
    Pst.Str(string("..."))
    Pst.Position(DEBUG_DYNAMIC_X, DEBUG_DYNAMIC_Y + 5)
    Pst.Dec(Down_Count)
    Pst.Str(string("..."))
    Pst.Position(DEBUG_DYNAMIC_X, DEBUG_DYNAMIC_Y + 6)
    Pst.Dec(UDdiff)
    Pst.Str(string("..."))
    Pst.Position(DEBUG_DYNAMIC_X, DEBUG_DYNAMIC_Y + 7)
    Pst.Dec(UDdiffavg)
    Pst.Str(string("..."))

    Pst.Position(DEBUG_DYNAMIC_X, DEBUG_DYNAMIC_Y + 8)
    Pst.Dec(Park_Flag)
    Pst.Str(string("..."))
    
    Pst.Position(DEBUG_DYNAMIC_X, DEBUG_DYNAMIC_Y + 9)
    Pst.Dec(WestSwitch)
    Pst.Str(string("..."))

    Pst.Position(DEBUG_DYNAMIC_X, DEBUG_DYNAMIC_Y + 10)
    Pst.Dec(EastSwitch)
    Pst.Str(string("..."))

    Pst.Position(DEBUG_DYNAMIC_X, DEBUG_DYNAMIC_Y + 11)
    Pst.Dec(threshold_flag)
    Pst.Str(string("..."))

    Pst.Position(DEBUG_DYNAMIC_X, DEBUG_DYNAMIC_Y + 12)
    Pst.Dec(moving_flag)
    Pst.Str(string("..."))

    Pst.Position(DEBUG_DYNAMIC_X, DEBUG_DYNAMIC_Y + 13)
    Pst.Str(@buffer)
    'Pst.Str(string("..."))
    return

PUB pause(Duration)
  if(Duration > 0)  
    waitcnt((clkfreq / 1_000 * Duration) + cnt)
  return